@@ -1,5 +1,5 @@
 """
-Panther Analysis Tool is a command line interface for writing,
+ Analysis Tool is a command line interface for writing,
 testing, and packaging policies/rules.
 Copyright (C) 2020 Panther Labs Inc
 
@@ -26,17 +26,20 @@ import logging
 import mimetypes
 import os
 import re
+import shutil
 import subprocess  # nosec
 import sys
+import tempfile
 import zipfile
 from collections import defaultdict
 from collections.abc import Mapping
 from dataclasses import asdict
 from datetime import datetime
+
 # Comment below disabling pylint checks is due to a bug in the CircleCi image with Pylint
 # It seems to be unable to import the distutils module, however the module is present and importable
 # in the Python Repl.
-from distutils.util import strtobool #pylint: disable=E0611, E0401
+from distutils.util import strtobool  # pylint: disable=E0611, E0401
 from fnmatch import fnmatch
 from importlib.abc import Loader
 from typing import Any, DefaultDict, Dict, Iterator, List, Set, Tuple, Type
@@ -45,6 +48,7 @@ from uuid import uuid4
 
 import botocore
 import requests
+import schema
 from dynaconf import Dynaconf, Validator
 from ruamel.yaml import YAML, SafeConstructor, constructor
 from ruamel.yaml import parser as YAMLParser
@@ -69,6 +73,7 @@ from panther_analysis_tool.rule import Detection, Rule
 from panther_analysis_tool.schemas import (
     DATA_MODEL_SCHEMA,
     GLOBAL_SCHEMA,
+    LOOKUP_TABLE_SCHEMA,
     PACK_SCHEMA,
     POLICY_SCHEMA,
     RULE_SCHEMA,
@@ -86,22 +91,25 @@ from panther_analysis_tool.util import get_client
 CONFIG_FILE = ".panther_settings.yml"
 DATA_MODEL_LOCATION = "./data_models"
 HELPERS_LOCATION = "./global_helpers"
-
+LUTS_LOCATION = "./lookup_tables"
 DATA_MODEL_PATH_PATTERN = "*data_models*"
+LUTS_PATH_PATTERN = "*lookup_tables*"
 HELPERS_PATH_PATTERN = "*/global_helpers"
 PACKS_PATH_PATTERN = "*/packs"
 POLICIES_PATH_PATTERN = "*policies*"
 QUERIES_PATH_PATTERN = "*queries*"
 RULES_PATH_PATTERN = "*rules*"
+TMP_HELPER_MODULE_LOCATION = os.path.join(tempfile.gettempdir(), "panther-path", "globals")
 
 DATAMODEL = "datamodel"
 DETECTION = "detection"
 GLOBAL = "global"
+LOOKUP_TABLE = "lookup_table"
 PACK = "pack"
 POLICY = "policy"
 QUERY = "scheduled_query"
-SCHEDULED_RULE = "scheduled_rule"
 RULE = "rule"
+SCHEDULED_RULE = "scheduled_rule"
 
 RESERVED_FUNCTIONS = (
     "alert_context",
@@ -119,6 +127,7 @@ VALID_SEVERITIES = ["INFO", "LOW", "MEDIUM", "HIGH", "CRITICAL"]
 SCHEMAS: Dict[str, Schema] = {
     DATAMODEL: DATA_MODEL_SCHEMA,
     GLOBAL: GLOBAL_SCHEMA,
+    LOOKUP_TABLE: LOOKUP_TABLE_SCHEMA,
     PACK: PACK_SCHEMA,
     POLICY: POLICY_SCHEMA,
     QUERY: SCHEDULED_QUERY_SCHEMA,
@@ -225,6 +234,7 @@ def load_analysis_specs(
                         for path_pattern in (
                             DATA_MODEL_PATH_PATTERN,
                             HELPERS_PATH_PATTERN,
+                            LUTS_PATH_PATTERN,
                             RULES_PATH_PATTERN,
                             PACKS_PATH_PATTERN,
                             POLICIES_PATH_PATTERN,
@@ -253,7 +263,7 @@ def load_analysis_specs(
                         try:
                             yield spec_filename, relative_path, yaml.load(spec_file_obj), None
                         except (YAMLParser.ParserError, YAMLScanner.ScannerError) as err:
-                            # recreate the yaml object and yeild the error
+                            # recreate the yaml object and yield the error
                             yaml = YAML(typ="safe")
                             yield spec_filename, relative_path, None, err
                 if fnmatch(filename, "*.json"):
@@ -308,7 +318,7 @@ def zip_analysis(args: argparse.Namespace) -> Tuple[int, str]:
         if return_code != 0:
             return return_code, ""
 
-    logging.info("Zipping analysis packs in %s to %s", args.path, args.out)
+    logging.info("Zipping analysis items in %s to %s", args.path, args.out)
     # example: 2019-08-05T18-23-25
     # The colon character is not valid in filenames.
     current_time = datetime.now().isoformat(timespec="seconds").replace(":", "-")
@@ -327,7 +337,7 @@ def zip_analysis(args: argparse.Namespace) -> Tuple[int, str]:
         files: Set[str] = set()
         for (file_name, f_path, spec, _) in list(
             load_analysis_specs(
-                [args.path, HELPERS_LOCATION, DATA_MODEL_LOCATION], args.ignore_files
+                [args.path, HELPERS_LOCATION, DATA_MODEL_LOCATION, LUTS_LOCATION], args.ignore_files
             )
         ):
             if file_name not in files:
@@ -372,7 +382,7 @@ def upload_analysis(args: argparse.Namespace) -> Tuple[int, str]:
             },
         }
 
-        logging.info("Uploading pack to Panther")
+        logging.info("Uploading items to Panther")
         response = client.invoke(
             FunctionName="panther-analysis-api",
             InvocationType="RequestResponse",
@@ -398,6 +408,284 @@ def upload_analysis(args: argparse.Namespace) -> Tuple[int, str]:
     return 0, ""
 
 
+def detection_info_query(args: argparse.Namespace, filter_by: str, object_list: list) -> Any:
+    # Queries analysis-api for information around detections. Arguments are what parameter in the
+    # API we want to filter on, and a list of values for that filter
+    query_list_payload = {"listDetections": {filter_by: object_list}}
+    client = get_client(args.aws_profile, "lambda")
+
+    query_info = client.invoke(
+        FunctionName="panther-analysis-api",
+        InvocationType="RequestResponse",
+        LogType="None",
+        Payload=json.dumps(query_list_payload),
+    )
+
+    if query_info["ResponseMetadata"]["HTTPStatusCode"] != 200:
+        logging.warning(
+            "Failed to search for associated queries, API error.\n\t status code: %s",
+            query_info["ResponseMetadata"]["HTTPStatusCode"],
+        )
+        return {}
+
+    analysis_api_response = json.loads(query_info["Payload"].read().decode("utf-8"))
+    analysis_api_json_response = json.loads(analysis_api_response["body"])
+
+    return analysis_api_json_response
+
+
+def get_analysis_id_by_query(args: argparse.Namespace, query_list: list) -> list:
+    # Retrieves analysis_ids associated with saved queries. Generally these are scheduled rules
+    analysis_json_output = detection_info_query(args, "scheduledQueries", query_list)
+
+    analysis_id_list = []
+    for detection in analysis_json_output.get("detections"):
+        analysis_id_list.append(detection.get("id"))
+
+    return analysis_id_list
+
+
+def get_query_by_analysis_id(args: argparse.Namespace, analysis_id_list: list) -> list:
+    # Retrieves saved queries associated with analysis_id
+    analysis_json_output = detection_info_query(args, "ids", analysis_id_list)
+    query_list = []
+    for detection in analysis_json_output.get("detections"):
+        for query in detection.get("scheduledQueries"):
+            query_list.append(query)
+
+    return query_list
+
+
+def confirm_analysis_exists(args: argparse.Namespace, analysis_id_list: list) -> list:
+    validation_payload = {"listDetections": {"ids": analysis_id_list}}
+    client = get_client(args.aws_profile, "lambda")
+
+    validation = client.invoke(
+        FunctionName="panther-analysis-api",
+        InvocationType="RequestResponse",
+        LogType="None",
+        Payload=json.dumps(validation_payload),
+    )
+
+    if validation["ResponseMetadata"]["HTTPStatusCode"] != 200:
+        logging.warning(
+            "Failed to validate analysis, API error.\n\t status code: %s",
+            validation["ResponseMetadata"]["HTTPStatusCode"],
+        )
+        return []
+
+    api_response = json.loads(validation["Payload"].read().decode("utf-8"))
+    json_response = api_response["body"]
+
+    json_output = json.loads(json_response)
+    # validate the API response matches what we passed in to avoid partial matching
+    # pylint: disable=consider-using-set-comprehension)
+    existing_ids = set([detection["id"] for detection in json_output["detections"]])
+    given_ids = set(analysis_id_list)
+    diff = given_ids.difference(existing_ids)
+    if diff:
+        for detection_id in diff:
+            logging.info("%s was not found, skipping...", detection_id)
+
+    return list(existing_ids.intersection(given_ids))
+
+
+def delete_queries(args: argparse.Namespace, query_list: list) -> Tuple[int, str]:
+    client = get_client(args.aws_profile, "lambda")
+
+    datalake_function = "panther-snowflake-api"
+    if args.athena_datalake:
+        datalake_function = "panther-athena-api"
+
+    # Delete function needs the query ID, required endpoint wont take a list
+    query_map = {}
+    for query in query_list:
+        payload = {"listSavedQueries": {"pageSize": 1, "name": query}}
+        list_response = client.invoke(
+            FunctionName=datalake_function,
+            InvocationType="RequestResponse",
+            LogType="None",
+            Payload=json.dumps(payload),
+        )
+        api_response = json.loads(list_response["Payload"].read().decode("utf-8"))
+        api_saved_query_response = api_response.get("savedQueries")
+        if len(api_saved_query_response) == 0:
+            logging.warning("%s was not found, skipping...", query)
+            continue
+        query_id = api_saved_query_response[0]["id"]
+        query_map[query] = query_id
+
+    # Now we have query ids, lets delete them
+
+    if len(query_map) > 0:
+        payload = {
+            "deleteSavedQueries": {
+                "ids": list(query_map.values()),
+                "userId": "00000000-0000-4000-8000-000000000000",
+                # The UserID is required by Panther for this API call, but we have no way of
+                # acquiring it, and it isn't used for anything. This is a valid UUID used by the
+                # Panther deployment tool to indicate this action was performed automatically.
+            }
+        }
+        delete_response = client.invoke(
+            FunctionName=datalake_function,
+            InvocationType="RequestResponse",
+            LogType="None",
+            Payload=json.dumps(payload),
+        )
+
+        if delete_response.get("ResponseMetadata").get("HTTPStatusCode") != 200:
+            error_payload = json.loads(list_response["Payload"].read().decode("utf-8"))
+            error_message = error_payload.get("errorMessage")
+            return 1, f"Error deleting queries, API error {error_message}"
+        logging.info("Queries %s have been deleted.", " ".join(list(query_map.keys())))
+        return 0, ""
+    return 1, "No queries left to delete, exiting"
+
+
+def delete_detections(args: argparse.Namespace, analysis_id_list: list) -> Tuple[int, str]:
+    client = get_client(args.aws_profile, "lambda")
+    payload: dict = {"deleteDetections": {"entries": []}}
+    for analysis_id in analysis_id_list:
+        payload["deleteDetections"]["entries"].append({"id": analysis_id})
+
+    response = client.invoke(
+        FunctionName="panther-analysis-api",
+        InvocationType="RequestResponse",
+        LogType="None",
+        Payload=json.dumps(payload),
+    )
+
+    response_str = response["Payload"].read().decode("utf-8")
+    response_payload = json.loads(response_str)
+
+    if response_payload.get("statusCode") != 200:
+        logging.warning(
+            "Failed to delete analysis.\n\tstatus code: %s\n\terror message: %s",
+            response_payload.get("statusCode", 0),
+            response_payload.get("errorMessage", response_payload.get("body")),
+        )
+        return 1, ""
+
+    logging.info("Detection(s) %s have been deleted.", " ".join(analysis_id_list))
+    return 0, ""
+
+
+def delete_router(args: argparse.Namespace) -> Tuple[int, str]:
+    # Routes all things delete to the functions they need to go to
+
+    # Get lists of analysis and queries from args
+    analysis_id_list = args.analysis_id
+    query_list = args.query_id
+
+    if len(analysis_id_list) == 0 and len(query_list) == 0:
+        logging.error("Must specify a list of analysis or queries to delete")
+        logging.error("Run panther_analysis_tool -h for help statement")
+        return 1, ""
+
+    # Look for associated queries / analysis
+    associated_analysis_id_list = []
+    associated_query_list = []
+    # If we get passed a list of queries look up the associated analysis
+    if len(query_list) > 0:
+        associated_analysis_id_list = get_analysis_id_by_query(args, query_list)
+
+    # Similar to above, if we get analysis, look for queries. Also ensure all analysis exist
+    # Query existence is validated in the query delete function
+    if len(analysis_id_list) > 0:
+        analysis_id_list = confirm_analysis_exists(args, analysis_id_list)
+        if len(analysis_id_list) > 0:
+            associated_query_list = get_query_by_analysis_id(args, analysis_id_list)
+
+    # Merge what we looked up with what was passed, removing dupes
+    analysis_id_list = list(set(analysis_id_list + associated_analysis_id_list))
+    query_list = list(set(associated_query_list + query_list))
+
+    if len(analysis_id_list) == 0 and len(query_list) == 0:
+        logging.warning("No matching analysis or queries found, exiting")
+        return 1, ""
+
+    # Unless explicitly bypassed, get user confirmation to delete
+    if not args.confirm_bypass:
+        if len(analysis_id_list) > 0:
+            analysis_id_string = " ".join(analysis_id_list)
+            logging.warning("You are about to delete detections %s", analysis_id_string)
+        if len(query_list) > 0:
+            associated_query_string = " ".join(query_list)
+            logging.warning("You are about to delete queries %s", associated_query_string)
+        confirm = input("Continue? (y/n) ")
+
+        if confirm.lower() != "y":
+            print("Cancelled")
+            return 0, ""
+
+    # After confirmation and validation then delete things
+    if len(query_list) > 0:
+        delete_queries(args, query_list)
+    if len(analysis_id_list) > 0:
+        delete_detections(args, analysis_id_list)
+
+    return 0, ""
+
+
+def parse_lookup_table(args: argparse.Namespace) -> dict:
+    """Validates and parses a Lookup Table spec file
+
+    Returns a dict representing the Lookup Table
+
+    Args:
+        args: The populated Argparse namespace with parsed command-line arguments.
+
+    Returns:
+        A dict representing the Lookup Table, empty when parsing fails
+    """
+
+    logging.info("Parsing the Lookup Table spec defined in %s", args.path)
+    with open(args.path, "r") as input_file:
+        try:
+            yaml = YAML(typ="safe")
+            lookup_spec = yaml.load(input_file)
+            logging.info("Successfully parse the Lookup Table file %s", args.path)
+        except (YAMLParser.ParserError, YAMLScanner.ScannerError) as err:
+            logging.error("Failed to parse the Lookup Table spec file %s", input_file)
+            logging.error(err)
+            return {}
+        try:
+            LOOKUP_TABLE_SCHEMA.validate(lookup_spec)
+            logging.info("Successfully validated the Lookup Table file %s", args.path)
+        except (
+            schema.SchemaError,
+            schema.SchemaMissingKeyError,
+            schema.SchemaWrongKeyError,
+            schema.SchemaForbiddenKeyError,
+            schema.SchemaUnexpectedTypeError,
+            schema.SchemaOnlyOneAllowedError,
+        ) as err:
+            logging.error("Invalid schema in the Lookup Table spec file %s", input_file)
+            logging.error(err)
+            return {}
+    return lookup_spec
+
+
+def test_lookup_table(args: argparse.Namespace) -> Tuple[int, str]:
+    """Validates a Lookup Table spec file
+
+    Returns 1 if the validation fails
+
+    Args:
+        args: The populated Argparse namespace with parsed command-line arguments.
+
+    Returns:
+        A tuple of return code and and empty string (to satisfy calling conventions)
+    """
+
+    logging.info("Validating the Lookup Table spec defined in %s", args.path)
+    lookup_spec = parse_lookup_table(args)
+    if not lookup_spec:
+        return 1, ""
+    return 0, ""
+
+
 def update_schemas(args: argparse.Namespace) -> Tuple[int, str]:
     """Updates managed schemas in a Panther deployment.
 
@@ -683,7 +971,7 @@ def test_analysis(args: argparse.Namespace) -> Tuple[int, list]:
     Returns:
         A tuple of the return code, and a list of tuples containing invalid specs and their error.
     """
-    logging.info("Testing analysis packs in %s\n", args.path)
+    logging.info("Testing analysis items in %s\n", args.path)
 
     ignored_files = args.ignore_files
     search_directories = [args.path]
@@ -694,6 +982,8 @@ def test_analysis(args: argparse.Namespace) -> Tuple[int, list]:
         "." + HELPERS_LOCATION,
         DATA_MODEL_LOCATION,
         "." + DATA_MODEL_LOCATION,
+        LUTS_LOCATION,
+        "." + LUTS_LOCATION,
     ):
         absolute_dir_path = os.path.abspath(os.path.join(args.path, directory))
         absolute_helper_path = os.path.abspath(directory)
@@ -736,12 +1026,10 @@ def test_analysis(args: argparse.Namespace) -> Tuple[int, list]:
         name: FakeDestination(destination_id=str(uuid4()), destination_display_name=name)
         for name in available_destinations
     }
-
     # import each data model, global, policy, or rule and run its tests
     # first import the globals
     #   add them sys.modules to be used by rule and/or policies tests
-    invalid_globals = setup_global_helpers(specs[GLOBAL])
-    invalid_specs.extend(invalid_globals)
+    setup_global_helpers(specs[GLOBAL])
 
     # then, setup data model dictionary to be used in rule/policy tests
     log_type_to_data_model, invalid_data_models = setup_data_models(specs[DATAMODEL])
@@ -758,21 +1046,49 @@ def test_analysis(args: argparse.Namespace) -> Tuple[int, list]:
     )
     invalid_specs.extend(invalid_detection)
 
+    # finally, validate pack defs
+    invalid_packs = validate_packs(specs)
+    invalid_specs.extend(invalid_packs)
+
+    # cleanup tmp global dir
+    cleanup_global_helpers(specs[GLOBAL])
+
     print_summary(args.path, len(specs[DETECTION]), failed_tests, invalid_specs)
     return int(bool(failed_tests or invalid_specs)), invalid_specs
 
 
-def setup_global_helpers(global_analysis: List[Any]) -> List[Any]:
-    invalid_specs = []
-    for analysis_spec_filename, dir_name, analysis_spec in global_analysis:
+def setup_global_helpers(global_analysis: List[Any]) -> None:
+    # ensure the directory does not exist, else clear it
+    cleanup_global_helpers(global_analysis)
+    os.makedirs(TMP_HELPER_MODULE_LOCATION)
+    # setup temp dir for globals
+    if TMP_HELPER_MODULE_LOCATION not in sys.path:
+        sys.path.append(TMP_HELPER_MODULE_LOCATION)
+    # place globals in temp dir
+    for _, dir_name, analysis_spec in global_analysis:
         analysis_id = analysis_spec["GlobalID"]
-        module, load_err = load_module(os.path.join(dir_name, analysis_spec["Filename"]))
-        # If the module could not be loaded, continue to the next
-        if load_err:
-            invalid_specs.append((analysis_spec_filename, load_err))
-            continue
-        sys.modules[analysis_id] = module
-    return invalid_specs
+        source = os.path.join(dir_name, analysis_spec["Filename"])
+        destination = os.path.join(TMP_HELPER_MODULE_LOCATION, f"{analysis_id}.py")
+        shutil.copyfile(source, destination)
+        # force reload of the module as necessary
+        if analysis_id in sys.modules:
+            logging.warning(
+                "module name collision: global (%s) has same name as a module in python path",
+                analysis_id,
+            )
+            importlib.reload(sys.modules[analysis_id])
+
+
+def cleanup_global_helpers(global_analysis: List[Any]) -> None:
+    # clear the modules from the modules cache
+    for _, _, analysis_spec in global_analysis:
+        analysis_id = analysis_spec["GlobalID"]
+        # delete the helpers that were added to sys.modules for testing
+        if analysis_id in sys.modules:
+            del sys.modules[analysis_id]
+    # ensure the directory does not exist, else clear it
+    if os.path.exists(TMP_HELPER_MODULE_LOCATION):
+        shutil.rmtree(TMP_HELPER_MODULE_LOCATION)
 
 
 def setup_data_models(data_models: List[Any]) -> Tuple[Dict[str, DataModel], List[Any]]:
@@ -879,6 +1195,40 @@ def setup_run_tests(  # pylint: disable=too-many-locals,too-many-arguments
     return failed_tests, invalid_specs
 
 
+def validate_packs(analysis_specs: Dict[str, List[Any]]) -> List[Any]:
+
+    invalid_specs = []
+    # first, setup dictionary of id to detection item
+    id_to_detection = {}
+    for analysis_type in analysis_specs:
+        for analysis_spec_filename, _, analysis_spec in analysis_specs[analysis_type]:
+            analysis_id = (
+                analysis_spec.get("PolicyID")
+                or analysis_spec.get("RuleID")
+                or analysis_spec.get("DataModelID")
+                or analysis_spec.get("GlobalID")
+                or analysis_spec.get("PackID")
+                or analysis_spec.get("QueryName")
+                or analysis_spec["LookupName"]
+            )
+            id_to_detection[analysis_id] = analysis_spec
+    for analysis_spec_filename, _, analysis_spec in analysis_specs[PACK]:
+        # validate each id in the pack def exists
+        pack_invalid_ids = []
+        for analysis_id in analysis_spec.get("PackDefinition", {}).get("IDs", []):
+            if analysis_id not in id_to_detection:
+                pack_invalid_ids.append(analysis_id)
+        if pack_invalid_ids:
+            invalid_specs.append(
+                (
+                    analysis_spec_filename,
+                    f"pack ({analysis_spec['PackID']}) definition includes item(s)"
+                    f" that do no exist ({', '.join(pack_invalid_ids)})",
+                )
+            )
+    return invalid_specs
+
+
 def print_summary(
     test_path: str, num_tests: int, failed_tests: Dict[str, list], invalid_specs: List[Any]
 ) -> None:
@@ -921,6 +1271,10 @@ def filter_analysis(
             logging.debug("auto-adding data model file %s", os.path.join(file_name))
             filtered_analysis.append((file_name, dir_name, analysis_spec))
             continue
+        if fnmatch(dir_name, LUTS_PATH_PATTERN):
+            logging.debug("auto-adding lookup table file %s", os.path.join(file_name))
+            filtered_analysis.append((file_name, dir_name, analysis_spec))
+            continue
         match = True
         for key, values in filters.items():
             spec_value = analysis_spec.get(key, "")
@@ -950,7 +1304,7 @@ def classify_analysis(
     # types of detections, meta types that can be zipped
     # or uploaded
     classified_specs: Dict[str, List[Any]] = dict()
-    for key in [DATAMODEL, DETECTION, GLOBAL, PACK, QUERY]:
+    for key in [DATAMODEL, DETECTION, LOOKUP_TABLE, GLOBAL, PACK, QUERY]:
         classified_specs[key] = []
 
     invalid_specs = []
@@ -1039,6 +1393,8 @@ def lookup_analysis_id(analysis_spec: Any, analysis_type: str) -> str:
         analysis_id = analysis_spec["DataModelID"]
     if analysis_type == GLOBAL:
         analysis_id = analysis_spec["GlobalID"]
+    if analysis_type == LOOKUP_TABLE:
+        analysis_id = analysis_spec["LookupName"]
     if analysis_type == PACK:
         analysis_id = analysis_spec["PackID"]
     if analysis_type == POLICY:
@@ -1266,6 +1622,13 @@ def setup_parser() -> argparse.ArgumentParser:
         "help": "The relative path to Panther policies and rules.",
         "required": False,
     }
+    lut_path_name = "--path"
+    lut_path_arg: Dict[str, Any] = {
+        "default": ".",
+        "type": str,
+        "help": "The relative path to a lookup table input file.",
+        "required": True,
+    }
     skip_test_name = "--skip-tests"
     skip_test_arg: Dict[str, Any] = {
         "action": "store_true",
@@ -1306,13 +1669,31 @@ def setup_parser() -> argparse.ArgumentParser:
         "help": "A destination name that may be returned by the destinations function. "
         "Repeat the argument to define more than one name.",
     }
+    analysis_id_name = "--analysis-id"
+    analysis_id_arg: Dict[str, Any] = {
+        "required": False,
+        "dest": "analysis_id",
+        "nargs": "+",
+        "help": "Space separated list of Rule or Policy IDs",
+        "type": str,
+        "default": [],
+    }
+    query_id_name = "--query-id"
+    query_id_arg: Dict[str, Any] = {
+        "required": False,
+        "dest": "query_id",
+        "nargs": "+",
+        "help": "Space separated list of Saved Queries",
+        "type": str,
+        "default": [],
+    }
 
     parser = argparse.ArgumentParser(
         description="Panther Analysis Tool: A command line tool for "
         + "managing Panther policies and rules.",
         prog="panther_analysis_tool",
     )
-    parser.add_argument("--version", action="version", version="panther_analysis_tool 0.10.6")
+    parser.add_argument("--version", action="version", version="panther_analysis_tool 0.14.1")
     parser.add_argument("--debug", action="store_true", dest="debug")
     subparsers = parser.add_subparsers()
 
@@ -1408,6 +1789,26 @@ def setup_parser() -> argparse.ArgumentParser:
     upload_parser.add_argument(available_destination_name, **available_destination_arg)
     upload_parser.set_defaults(func=upload_analysis)
 
+    delete_parser = subparsers.add_parser(
+        "delete", help="Delete policies, rules, or saved queries from a Panther deployment"
+    )
+    delete_parser.add_argument(
+        "--no-confirm",
+        help="Skip manual confirmation of deletion",
+        action="store_true",
+        dest="confirm_bypass",
+    )
+    delete_parser.add_argument(
+        "--athena-datalake",
+        help="Instance DataLake is backed by Athena",
+        action="store_true",
+        dest="athena_datalake",
+    )
+    delete_parser.add_argument(aws_profile_name, **aws_profile_arg)
+    delete_parser.add_argument(analysis_id_name, **analysis_id_arg)
+    delete_parser.add_argument(query_id_name, **query_id_arg)
+    delete_parser.set_defaults(func=delete_router)
+
     update_custom_schemas_parser = subparsers.add_parser(
         "update-custom-schemas", help="Update or create custom schemas on a Panther deployment."
     )
@@ -1417,6 +1818,13 @@ def setup_parser() -> argparse.ArgumentParser:
     update_custom_schemas_parser.add_argument(path_name, **custom_schemas_path_arg)
     update_custom_schemas_parser.set_defaults(func=update_custom_schemas)
 
+    test_lookup_table_parser = subparsers.add_parser(
+        "test-lookup-table", help="Validate a Lookup Table spec file."
+    )
+    test_lookup_table_parser.add_argument(aws_profile_name, **aws_profile_arg)
+    test_lookup_table_parser.add_argument(lut_path_name, **lut_path_arg)
+    test_lookup_table_parser.set_defaults(func=test_lookup_table)
+
     zip_parser = subparsers.add_parser(
         "zip", help="Create an archive of local policies and rules for uploading to Panther."
     )
