@@ -28,9 +28,10 @@ RESOURCE_TYPE_REGEX = Regex(
     r"\.Policy|IAM\.Role|IAM\.RootUser|IAM\.User|KMS\.Key|Lambda\.Function|PasswordPolicy|RDS"
     r"\.Instance|Redshift\.Cluster|S3\.Bucket|WAF\.Regional\.WebACL|WAF\.WebACL)$"
 )
+# pylint: disable=line-too-long
 LOG_TYPE_REGEX = Regex(
-    r"^(Apache\.AccessCombined|Apache\.AccessCommon|Atlassian\.Audit|AWS\.ALB"
-    r"|AWS\.AuroraMySQLAudit"
+    r"^(Apache\.AccessCombined|Apache\.AccessCommon|Asana\.Audit"
+    r"|Atlassian\.Audit|AWS\.ALB|AWS\.AuroraMySQLAudit"
     r"|AWS\.CloudTrail|AWS\.CloudTrailDigest|AWS\.CloudTrailInsight|AWS\.CloudWatchEvents"
     r"|AWS\.GuardDuty|AWS\.S3ServerAccess|AWS\.VPCDns|AWS\.VPCFlow|AWS\.WAFWebACL|Box\.Event"
     r"|CiscoUmbrella\.CloudFirewall|CiscoUmbrella\.DNS|CiscoUmbrella\.IP|CiscoUmbrella\."
@@ -50,8 +51,8 @@ LOG_TYPE_REGEX = Regex(
     r"|Microsoft365\.Audit\.General|Microsoft365\.Audit\.SharePoint|Microsoft365\.DLP\.All|"
     r"Nginx\.Access|Okta\.SystemLog|OneLogin\.Events|Osquery\.Batch|Osquery\.Differential|"
     r"Osquery\.Snapshot|Osquery\.Status|OSSEC\.EventInfo|OnePassword\.ItemUsage|OnePassword"
-    r"\.SignInAttempt|Salesforce\.Login|Salesforce\.LoginAs|Salesforce\.Logout|Salesforce\."
-    r"URI|Slack\.AccessLogs|Slack\.AuditLogs|Slack\.IntegrationLogs|Sophos\.Central|Suricata"
+    r"\.SignInAttempt|Panther\.Audit|Salesforce\.Login|Salesforce\.LoginAs|Salesforce\.Logout|"
+    r"Salesforce\.URI|Slack\.AccessLogs|Slack\.AuditLogs|Slack\.IntegrationLogs|Sophos\.Central|Suricata"
     r"\.Anomaly|Suricata\.DNS|Syslog\.RFC3164|Syslog\.RFC5424|Zeek\.DNS|Zendesk\.Audit|Zendesk"
     r"\.AuditLog|Zoom\.Activity|Zoom\.Operation|"
     r"Custom\.([A-Z][A-Za-z0-9]*)(\.[A-Z][A-Za-z0-9]*){0,5})$"
@@ -60,7 +61,14 @@ LOG_TYPE_REGEX = Regex(
 TYPE_SCHEMA = Schema(
     {
         "AnalysisType": Or(
-            "datamodel", "global", "pack", "policy", "rule", "scheduled_rule", "scheduled_query"
+            "datamodel",
+            "global",
+            "pack",
+            "policy",
+            "rule",
+            "scheduled_rule",
+            "scheduled_query",
+            "lookup_table",
         ),
     },
     ignore_extra_keys=True,
@@ -190,3 +198,28 @@ SCHEDULED_QUERY_SCHEMA = Schema(
     },
     ignore_extra_keys=False,
 )  # Prevent user typos on optional fields
+
+LOOKUP_TABLE_SCHEMA = Schema(
+    {
+        "AnalysisType": Or("lookup_table"),
+        "LookupName": str,
+        "Enabled": bool,
+        Or("DataFile", "Refresh"): Or(
+            str,
+            {
+                "RoleARN": str,
+                "ObjectPath": str,
+                Optional("PeriodMinutes"): int,
+                Optional("ObjectKMSKey"): str,
+            },
+        ),
+        "Schema": str,
+        "LogTypeMap": {
+            "PrimaryKey": str,
+            "AssociatedLogTypes": [{"LogType": str, Optional("Selectors"): [str]}],
+        },
+        Optional("Description"): str,
+        Optional("Reference"): str,
+    },
+    ignore_extra_keys=False,
+)  # Prevent user typos on optional fields
